@@ -90,6 +90,7 @@ module Data.Enumerator.Binary
 	-- ** Unsorted
 	, require
 	, isolate
+	, isolateWhile
 	, splitWhen
 	
 	) where
@@ -108,9 +109,8 @@ import           Data.Word (Word8)
 import qualified System.IO as IO
 import           System.IO.Error (isEOFError)
 
-import           Data.Enumerator hiding ( head, drop, iterateM, repeatM, replicateM
-                                        , generateM, filterM, consume, foldM
-                                        , concatMapM)
+import           Data.Enumerator.Internal
+import           Data.Enumerator (isEOF, throwError, tryIO)
 import qualified Data.Enumerator.List as EL
 
 -- | Consume the entire input stream with a strict left fold, one byte
@@ -194,7 +194,7 @@ concatMapM f = checkDone (continue . step) where
 	loop k (x:xs) = do
 		fx <- lift (f x)
 		k (Chunks [fx]) >>==
-			checkDoneEx (Chunks [B.pack xs]) (\k' -> loop k' xs)
+			checkDoneEx (Chunks [B.pack xs]) (`loop` xs)
 
 -- | Similar to 'Data.Enumerator.Binary.concatMap', but with a stateful step
 -- function.
@@ -326,7 +326,7 @@ take n = continue (loop id n) where
 		len = toInteger (BL.length lazy)
 		
 		iter = if len < n'
-			then continue (loop (acc . (BL.append lazy)) (n' - len))
+			then continue (loop (acc . BL.append lazy) (n' - len))
 			else let
 				(xs', extra) = BL.splitAt (fromInteger n') lazy
 				in yield (acc xs') (toChunks extra)
@@ -343,7 +343,7 @@ takeWhile p = continue (loop id) where
 		lazy = BL.fromChunks xs
 		(xs', extra) = BL.span p lazy
 		iter = if BL.null extra
-			then continue (loop (acc . (BL.append lazy)))
+			then continue (loop (acc . BL.append lazy))
 			else yield (acc xs') (toChunks extra)
 	loop acc EOF = yield (acc BL.empty) EOF
 
@@ -355,7 +355,7 @@ consume = continue (loop id) where
 	loop acc (Chunks []) = continue (loop acc)
 	loop acc (Chunks xs) = iter where
 		lazy = BL.fromChunks xs
-		iter = continue (loop (acc . (BL.append lazy)))
+		iter = continue (loop (acc . BL.append lazy))
 	loop acc EOF = yield (acc BL.empty) EOF
 
 -- | Pass input from a stream through two iteratees at once. Excess input is
@@ -665,7 +665,7 @@ require n = continue (loop id n) where
 		lazy = BL.fromChunks xs
 		len = toInteger (BL.length lazy)
 		iter = if len < n'
-			then continue (loop (acc . (BL.append lazy)) (n' - len))
+			then continue (loop (acc . BL.append lazy) (n' - len))
 			else yield () (toChunks (acc lazy))
 	loop _ _ EOF = throwError (Exc.ErrorCall "require: Unexpected EOF")
 
@@ -686,10 +686,27 @@ isolate n (Continue k) = continue loop where
 			then k (Chunks xs) >>== isolate (n - len)
 			else let
 				(s1, s2) = BL.splitAt (fromInteger n) lazy
-				in k (toChunks s1) >>== (\step -> yield step (toChunks s2))
-	loop EOF = k EOF >>== (\step -> yield step EOF)
+				in k (toChunks s1) >>== (`yield` toChunks s2)
+	loop EOF = k EOF >>== (`yield` EOF)
 isolate n step = drop n >> return step
 
+-- | @'isolateWhile' p@ reads bytes from the stream until /p/ is false, and
+-- passes them to its iteratee. If the iteratee finishes early, bytes
+-- continue to be consumed from the outer stream until /p/ is false.
+--
+-- Since: 0.4.16
+isolateWhile :: Monad m => (Word8 -> Bool) -> Enumeratee B.ByteString B.ByteString m b
+isolateWhile p (Continue k) = continue loop where
+	loop (Chunks []) = continue loop
+	loop (Chunks xs) = iter where
+		lazy = BL.fromChunks xs
+		(s1, s2) = BL.span p lazy
+		iter = if BL.null s2
+			then k (Chunks xs) >>== isolateWhile p
+			else k (toChunks s1) >>== (`yield` toChunks s2)
+	loop EOF = k EOF >>== (`yield` EOF)
+isolateWhile p step = Data.Enumerator.Binary.dropWhile p >> return step
+
 -- | Split on bytes satisfying a given predicate.
 --
 -- Since: 0.4.8
@@ -774,7 +791,7 @@ enumHandleRange bufferSize offset count h s = seek >> enum where
 			if B.null bytes
 				then continue k
 				else feed bytes
-		feed bs = k (Chunks [bs]) >>== loop (n - (toInteger (B.length bs)))
+		feed bs = k (Chunks [bs]) >>== loop (n - toInteger (B.length bs))
 		in if rem <= 0
 			then continue k
 			else keepGoing
