@@ -79,6 +79,7 @@ module Data.Enumerator.List
 	-- ** Unsorted
 	, require
 	, isolate
+	, isolateWhile
 	, splitWhen
 	
 	) where
@@ -91,9 +92,8 @@ import qualified Data.List as L
 import           Data.Monoid (mappend)
 import qualified Data.Set
 
-import           Data.Enumerator hiding ( concatMapM, iterateM, replicateM, head, drop
-                                        , foldM, repeatM, generateM, filterM, consume
-                                        , length)
+import           Data.Enumerator (sequence, throwError)
+import           Data.Enumerator.Internal
 
 -- | Consume the entire input stream with a strict left fold, one element
 -- at a time.
@@ -160,7 +160,7 @@ concatMapM f = checkDone (continue . step) where
 	loop k (x:xs) = do
 		fx <- lift (f x)
 		k (Chunks fx) >>==
-			checkDoneEx (Chunks xs) (\k' -> loop k' xs)
+			checkDoneEx (Chunks xs) (`loop` xs)
 
 -- | @'Data.Enumerator.List.concatMap' f@ applies /f/ to each input element
 -- and feeds the resulting outputs to the inner iteratee.
@@ -676,10 +676,24 @@ isolate n (Continue k) = continue loop where
 		| len xs <= n = k (Chunks xs) >>== isolate (n - len xs)
 		| otherwise = let
 			(s1, s2) = L.genericSplitAt n xs
-			in k (Chunks s1) >>== (\step -> yield step (Chunks s2))
-	loop EOF = k EOF >>== (\step -> yield step EOF)
+			in k (Chunks s1) >>== (`yield` Chunks s2)
+	loop EOF = k EOF >>== (`yield` EOF)
 isolate n step = drop n >> return step
 
+-- | @'isolateWhile' p@ reads elements from the stream until /p/ is false, and
+-- passes them to its iteratee. If the iteratee finishes early, elements
+-- continue to be consumed from the outer stream until /p/ is false.
+--
+-- Since: 0.4.16
+isolateWhile :: Monad m => (a -> Bool) -> Enumeratee a a m b
+isolateWhile p (Continue k) = continue loop where
+	loop (Chunks []) = continue loop
+	loop (Chunks xs) = case Prelude.span p xs of
+		(_, []) -> k (Chunks xs) >>== isolateWhile p
+		(s1, s2) -> k (Chunks s1) >>== (`yield` Chunks s2)
+	loop EOF = k EOF >>== (`yield` EOF)
+isolateWhile p step = Data.Enumerator.List.dropWhile p >> return step
+
 -- | Split on elements satisfying a given predicate.
 --
 -- Since: 0.4.8
